@@ -28,7 +28,7 @@ import logging
 import jhbuild.moduleset
 import jhbuild.frontends
 from jhbuild.errors import UsageError, FatalError, CommandError
-from jhbuild.commands import Command, register_command
+from jhbuild.commands import Command, BuildCommand, register_command
 
 from jhbuild.config import parse_relative_time
 
@@ -63,7 +63,7 @@ class cmd_update(Command):
         module_set = jhbuild.moduleset.load(config)
         module_list = module_set.get_module_list(args or config.modules,
                 config.skip, tags=config.tags,
-                ignore_suggests=config.ignore_suggests)
+                include_suggests=not config.ignore_suggests)
         # remove modules up to startat
         if options.startat:
             while module_list and module_list[0].name != options.startat:
@@ -151,57 +151,7 @@ class cmd_cleanone(Command):
 register_command(cmd_cleanone)
 
 
-def check_bootstrap_updateness(config):
-    '''Check install date of bootstrap modules, and compare them to
-       the bootstrap moduleset file last modification date.
-    '''
-    try:
-        module_set = jhbuild.moduleset.load(config, uri = 'bootstrap')
-    except:
-        # failed to get bootstrap moduleset, silently ignore.
-        return
-    packagedb = module_set.packagedb
-
-    max_install_date = max([
-            packagedb.installdate(module.name)
-            for module in module_set.modules.values()])
-
-    if max_install_date is None:
-        # bootstrap module has never been built; probably the user doesn't want
-        # to use it
-        return
-
-    updated_modules = []
-    for module in module_set.modules.values():
-        pkg = packagedb.get(module.name)
-        if pkg is None:
-            continue
-        p_version = pkg.version
-        if p_version != module.get_revision():
-            updated_modules.append(module.name)
-
-    if not config.modulesets_dir:
-        return
-    bootstrap_uri = os.path.join(config.modulesets_dir, 'bootstrap.modules')
-    if not os.path.exists(bootstrap_uri):
-        return
-    bootstrap_mtime = os.stat(bootstrap_uri)[stat.ST_MTIME]
-
-    if max_install_date <= bootstrap_mtime:
-        # general note, to cover added modules
-        logging.info(
-                _('bootstrap moduleset has been updated since the last time '\
-                  'you used it, perhaps you should run jhbuild bootstrap.'))
-
-    if updated_modules:
-        # note about updated modules
-        logging.info(
-                _('some bootstrap modules have been updated, '\
-                  'perhaps you should update them: %s.') % \
-                  ', '.join(updated_modules))
-
-
-class cmd_build(Command):
+class cmd_build(BuildCommand):
     doc = N_('Update and compile all modules (the default)')
 
     name = 'build'
@@ -263,20 +213,23 @@ class cmd_build(Command):
             make_option('--min-age', metavar='TIME-SPEC',
                         action='store', dest='min_age', default=None,
                         help=_('skip modules installed less than the given time ago')),
+            make_option('--nodeps',
+                        action='store_false', dest='check_sysdeps', default=None,
+                        help=_('ignore missing system dependencies')),
             ])
 
     def run(self, config, options, args, help=None):
         config.set_from_cmdline_options(options)
 
-        if not config.quiet_mode:
-            check_bootstrap_updateness(config)
-
         module_set = jhbuild.moduleset.load(config)
         modules = args or config.modules
-        module_list = module_set.get_module_list(modules,
-                config.skip, tags = config.tags,
-                include_optional_modules=options.build_optional_modules,
-                ignore_suggests=config.ignore_suggests)
+        full_module_list = module_set.get_full_module_list \
+                               (modules, config.skip,
+                                include_suggests=not config.ignore_suggests,
+                                include_afters=options.build_optional_modules)
+        full_module_list = module_set.remove_tag_modules(full_module_list,
+                                                         config.tags)
+        module_list = module_set.remove_system_modules(full_module_list)
         # remove modules up to startat
         if options.startat:
             while module_list and module_list[0].name != options.startat:
@@ -289,13 +242,24 @@ class cmd_build(Command):
                     _('requested module is in the ignore list, nothing to do.'))
             return 0
 
+        module_state = module_set.get_system_modules(full_module_list)
+        if (config.check_sysdeps
+            and not self.required_system_dependencies_installed(module_state)):
+            self.print_system_dependencies(module_state)
+            raise FatalError(_('Required system dependencies not installed. '
+                               'Install using the command %(cmd)s or to '
+                               'ignore system dependencies use command-line '
+                               'option %(opt)s' \
+                               % {'cmd' : "'jhbuild sysdeps --install'",
+                                  'opt' : '--nodeps'}))
+
         build = jhbuild.frontends.get_buildscript(config, module_list, module_set=module_set)
         return build.build()
 
 register_command(cmd_build)
 
 
-class cmd_buildone(Command):
+class cmd_buildone(BuildCommand):
     doc = N_('Update and compile one or more modules')
 
     name = 'buildone'
@@ -341,9 +305,6 @@ class cmd_buildone(Command):
     def run(self, config, options, args, help=None):
         config.set_from_cmdline_options(options)
 
-        if not config.quiet_mode:
-            check_bootstrap_updateness(config)
-
         module_set = jhbuild.moduleset.load(config)
         module_list = []
         for modname in args:
@@ -356,8 +317,8 @@ class cmd_buildone(Command):
                 from jhbuild.modtypes.autotools import AutogenModule
                 module = AutogenModule(modname, default_repo.branch(modname))
                 module.config = config
-                logging.info(_('module "%s" does not exist, created automatically using repository "%s"') % \
-                                (modname, default_repo.name))
+                logging.info(_('module "%(modname)s" does not exist, created automatically using repository "%(reponame)s"') % \
+                             {'modname': modname, 'reponame': default_repo.name})
             module_list.append(module)
 
         if not module_list:
@@ -494,17 +455,16 @@ class cmd_list(Command):
         config.set_from_cmdline_options(options)
         module_set = jhbuild.moduleset.load(config)
         if options.startat and options.list_all_modules:
-            raise UsageError(_('Conflicting options specified '
-                               '(\'%s\' and \'%s\')') % \
-                               ('--start-at', '--all-modules'))
+            raise UsageError(_('Conflicting options specified (\'--start-at\' and \'--all-modules\')'))
 
         if options.list_all_modules:
             module_list = module_set.modules.values()
         else:
-            module_list = module_set.get_module_list(args or config.modules,
-                                config.skip, tags = config.tags,
-                                include_optional_modules = options.list_optional_modules,
-                                ignore_suggests=config.ignore_suggests)
+            module_list = module_set.get_module_list \
+                              (args or config.modules, config.skip,
+                               tags=config.tags,
+                               include_suggests= not config.ignore_suggests,
+                               include_afters=options.list_optional_modules)
 
         # remove modules up to startat
         if options.startat:
