@@ -19,6 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os
+import os.path
 import re
 import sys
 import traceback
@@ -37,15 +38,14 @@ if sys.platform.startswith('win'):
 __all__ = [ 'Config' ]
 
 _defaults_file = os.path.join(os.path.dirname(__file__), 'defaults.jhbuildrc')
-_default_jhbuildrc = os.path.join(os.environ['HOME'], '.jhbuildrc')
 
 _known_keys = [ 'moduleset', 'modules', 'skip', 'tags', 'prefix',
                 'partial_build', 'checkoutroot', 'buildroot', 'top_builddir',
                 'autogenargs', 'makeargs', 'nice_build', 'jobs',
                 'installprog', 'repos', 'branches', 'noxvfb', 'xvfbargs',
                 'builddir_pattern', 'module_autogenargs', 'module_makeargs',
-                'interact', 'buildscript', 'nonetwork',
-                'nobuild', 'makeclean', 'makecheck', 'module_makecheck',
+                'interact', 'buildscript', 'nonetwork', 'nobuild',
+                'noinstall', 'makeclean', 'makecheck', 'module_makecheck',
                 'use_lib64', 'tinderbox_outputdir', 'sticky_date',
                 'tarballdir', 'pretty_print', 'svn_program', 'makedist',
                 'makedistcheck', 'nonotify', 'notrayicon', 'cvs_program',
@@ -61,7 +61,7 @@ _known_keys = [ 'moduleset', 'modules', 'skip', 'tags', 'prefix',
                 'module_mirror_policy', 'dvcs_mirror_dir', 'build_targets',
                 'cmakeargs', 'module_cmakeargs', 'print_command_pattern',
                 'static_analyzer', 'module_static_analyzer', 'static_analyzer_template', 'static_analyzer_outputdir',
-
+                'check_sysdeps',
                 # Internal only keys (propagated from command line options)
                 '_internal_noautogen',
                 ]
@@ -150,7 +150,7 @@ def parse_relative_time(s):
 class Config:
     _orig_environ = None
 
-    def __init__(self, filename=_default_jhbuildrc):
+    def __init__(self, filename):
         self._config = {
             '__file__': _defaults_file,
             'addpath':  addpath,
@@ -195,12 +195,39 @@ class Config:
         except:
             traceback.print_exc()
             raise FatalError(_('could not load config defaults'))
-        self._config['__file__'] = filename
-        self.filename = filename
-        if not os.path.exists(filename):
-            raise FatalError(_('could not load config file, %s is missing') % filename)
 
-        self.load()
+        old_config = os.path.join(os.path.expanduser('~'), '.jhbuildrc')
+        new_config = os.path.join \
+                         (os.environ.get \
+                             ('XDG_CONFIG_HOME',
+                              os.path.join(os.path.expanduser('~'), '.config')),
+                          'jhbuildrc')
+
+        if filename:
+            if not os.path.exists(filename):
+                raise FatalError(_('could not load config file, %s is missing') % filename)
+        else:
+            if os.path.isfile(old_config) \
+                and not os.path.islink(old_config) \
+                and os.path.isfile(new_config) \
+                and not os.path.islink(new_config):
+                raise FatalError(_('The default location of the configuration '
+                                   'file has changed. Please move %(old_path)s'
+                                   ' to %(new_path)s.' \
+                                   % {'old_path': old_config,
+                                      'new_path': new_config}))
+            if os.path.exists(new_config):
+                filename = new_config
+            elif os.path.exists(old_config):
+                filename = old_config
+
+        if filename:
+            self._config['__file__'] = filename
+            self.filename = filename
+        else:
+            self._config['__file__'] = new_config
+            self.filename = new_config
+        self.load(filename)
         self.setup_env()
 
     def reload(self):
@@ -216,18 +243,19 @@ class Config:
             traceback.print_exc()
             raise FatalError(_('Could not include config file (%s)') % filename)
 
-    def load(self):
+    def load(self, filename=None):
         config = self._config
-        try:
-            execfile(self.filename, config)
-        except Exception, e:
-            if isinstance(e, FatalError):
-                # raise FatalErrors back, as it means an error in include()
-                # and it will print a traceback, and provide a meaningful
-                # message.
-                raise e
-            traceback.print_exc()
-            raise FatalError(_('could not load config file'))
+        if filename:
+            try:
+                execfile(filename, config)
+            except Exception, e:
+                if isinstance(e, FatalError):
+                    # raise FatalErrors back, as it means an error in include()
+                    # and it will print a traceback, and provide a meaningful
+                    # message.
+                    raise e
+                traceback.print_exc()
+                raise FatalError(_('could not load config file'))
 
         if not config.get('quiet_mode'):
             unknown_keys = []
@@ -272,7 +300,8 @@ class Config:
                          'tinderbox_outputdir', 'tarballdir', 'copy_dir',
                          'jhbuildbot_slaves_dir', 'jhbuildbot_dir',
                          'jhbuildbot_mastercfg', 'modulesets_dir',
-                         'dvcs_mirror_dir', 'static_analyzer_outputdir'):
+                         'dvcs_mirror_dir', 'static_analyzer_outputdir',
+                         'prefix'):
             if config.get(path_key):
                 config[path_key] = os.path.expanduser(config[path_key])
 
@@ -307,6 +336,19 @@ class Config:
                 self.use_local_modulesets = False
             self.modulesets_dir = None
 
+        if self.buildroot and not os.path.isabs(self.buildroot):
+            raise FatalError(_('%s must be an absolute path') % 'buildroot')
+        if not os.path.isabs(self.checkoutroot):
+            raise FatalError(_('%s must be an absolute path') % 'checkoutroot')
+        if not os.path.isabs(self.prefix):
+            raise FatalError(_('%s must be an absolute path') % 'prefix')
+        if not os.path.isabs(self.tarballdir):
+            raise FatalError(_('%s must be an absolute path') % 'tarballdir')
+        if (self.tinderbox_outputdir and
+            not os.path.isabs(self.tinderbox_outputdir)):
+            raise FatalError(_('%s must be an absolute path') %
+                             'tinderbox_outputdir')
+
     def get_original_environment(self):
         return self._orig_environ
 
@@ -412,15 +454,23 @@ class Config:
         addpath('XCURSOR_PATH', xcursordir)
 
         # GST_PLUGIN_PATH
-        for gst in ('gstreamer-1.0', 'gstreamer-0.10'):
-            gstplugindir = os.path.join(self.libdir , gst)
-            if os.path.exists(gstplugindir):
-                addpath('GST_PLUGIN_PATH', gstplugindir)
+        gstplugindir = os.path.join(self.libdir , 'gstreamer-0.10')
+        if os.path.exists(gstplugindir):
+            addpath('GST_PLUGIN_PATH', gstplugindir)
+
+        # GST_PLUGIN_PATH_1_0
+        gstplugindir = os.path.join(self.libdir , 'gstreamer-1.0')
+        if os.path.exists(gstplugindir):
+            addpath('GST_PLUGIN_PATH_1_0', gstplugindir)
 
         # GST_REGISTRY
-        gstregistry = os.path.join(self.prefix, '_jhbuild', 'gstreamer.registry')
+        gstregistry = os.path.join(self.prefix, '_jhbuild', 'gstreamer-0.10.registry')
         addpath('GST_REGISTRY', gstregistry)
 
+        # GST_REGISTRY_1_0
+        gstregistry = os.path.join(self.prefix, '_jhbuild', 'gstreamer-1.0.registry')
+        addpath('GST_REGISTRY_1_0', gstregistry)
+
         # ACLOCAL_PATH
         aclocalpath = os.path.join(self.prefix, 'share', 'aclocal')
         addpath('ACLOCAL_PATH', aclocalpath)
@@ -457,10 +507,14 @@ class Config:
         python_bin = os.environ.get('PYTHON', 'python')
         try:
             pythonversion = 'python' + get_output([python_bin, '-c',
-                'import sys; print ".".join([str(x) for x in sys.version_info[:2]])'],
+                'import sys; print(".".join([str(x) for x in sys.version_info[:2]]))'],
                 get_stderr = False).strip()
         except CommandError:
             pythonversion = 'python' + str(sys.version_info[0]) + '.' + str(sys.version_info[1])
+            if 'PYTHON' in os.environ:
+                logging.warn(_('Unable to determine python version using the '
+                               'PYTHON environment variable (%s). Using default "%s"')
+                             % (os.environ['PYTHON'], pythonversion))
 
         # In Python 2.6, site-packages got replaced by dist-packages, get the
         # actual value by asking distutils
@@ -468,10 +522,14 @@ class Config:
         try:
             python_packages_dir = get_output([python_bin, '-c',
                 'import os, distutils.sysconfig; '\
-                'print distutils.sysconfig.get_python_lib(prefix="%s").split(os.path.sep)[-1]' % self.prefix],
+                'print(distutils.sysconfig.get_python_lib(prefix="%s").split(os.path.sep)[-1])' % self.prefix],
                 get_stderr=False).strip()
         except CommandError:
             python_packages_dir = 'site-packages'
+            if 'PYTHON' in os.environ:
+                logging.warn(_('Unable to determine python site-packages directory using the '
+                               'PYTHON environment variable (%s). Using default "%s"')
+                             % (os.environ['PYTHON'], python_packages_dir))
             
         if self.use_lib64:
             pythonpath = os.path.join(self.prefix, 'lib64', pythonversion, python_packages_dir)
@@ -603,6 +661,9 @@ class Config:
             except ValueError:
                 raise FatalError(_('Failed to parse \'min_age\' relative '
                                    'time'))
+        if (hasattr(options, 'check_sysdeps') and
+            options.check_sysdeps is not None):
+            self.check_sysdeps = options.check_sysdeps
 
     def __setattr__(self, k, v):
         '''Override __setattr__ for additional checks on some options.'''
