@@ -160,6 +160,15 @@ class Package:
     def __repr__(self):
         return "<%s '%s'>" % (self.__class__.__name__, self.name)
 
+    def eval_args(self, args):
+        args = args.replace('${prefix}', self.config.prefix)
+        libsubdir = 'lib'
+        if self.config.use_lib64:
+            libsubdir = 'lib64'
+        libdir = os.path.join(self.config.prefix, libsubdir)
+        args = args.replace('${libdir}', libdir)
+        return args
+
     def get_extra_env(self):
         return self.config.module_extra_env.get(self.name)
     extra_env = property(get_extra_env)
@@ -225,11 +234,14 @@ them into the prefix."""
                 else:
                     os.mkdir(dest_path)
                 num_copied += self._process_install_files(installroot, src_path, prefix)
-                os.rmdir(src_path)
+                if os.path.islink(src_path):
+                    os.unlink(src_path)
+                else:
+                    os.rmdir(src_path)
             else:
                 num_copied += 1
                 try:
-                    os.rename(src_path, dest_path)
+                    fileutils.rename(src_path, dest_path)
                 except OSError, e:
                     logging.error(_('Failed to rename %(src)r to %(dest)r: %(msg)s') %
                                   {'src': src_path,
@@ -293,7 +305,7 @@ them into the prefix."""
             if os.path.exists(broken_name):
                 assert broken_name.startswith(buildscript.config.top_builddir)
                 shutil.rmtree(broken_name)
-            os.rename(destdir, broken_name)
+            fileutils.rename(destdir, broken_name)
         else:
             assert destdir.startswith(buildscript.config.prefix)
             os.rmdir(destdir)
@@ -428,6 +440,7 @@ them into the prefix."""
         instance.branch = get_branch(node, repositories, default_repo, config)
         instance.dependencies, instance.after, instance.suggests = get_dependencies(node)
         instance.supports_parallel_build = (node.getAttribute('supports-parallel-builds') != 'no')
+        instance.config = config
         pkg_config = find_first_child_node_content(node, 'pkg-config')
         if pkg_config != '':
             instance.pkg_config = pkg_config
