@@ -51,6 +51,7 @@ class AutogenModule(Package, DownloadableModule):
                  makeinstallargs='',
                  supports_non_srcdir_builds=True,
                  skip_autogen=False,
+                 skip_install_phase=False,
                  autogen_sh='autogen.sh',
                  makefile='Makefile',
                  autogen_template=None,
@@ -62,6 +63,7 @@ class AutogenModule(Package, DownloadableModule):
         self.makeinstallargs = makeinstallargs
         self.supports_non_srcdir_builds = supports_non_srcdir_builds
         self.skip_autogen = skip_autogen
+        self.skip_install_phase = skip_install_phase
         self.autogen_sh = autogen_sh
         self.makefile = makefile
         self.autogen_template = autogen_template
@@ -118,7 +120,8 @@ class AutogenModule(Package, DownloadableModule):
         buildscript.set_action(_('Configuring'), self)
 
         srcdir = self.get_srcdir(buildscript)
-        if not os.path.exists(os.path.join(srcdir, self.autogen_sh)):
+        if self.autogen_sh == 'autogen.sh' and \
+        not os.path.exists(os.path.join(srcdir, self.autogen_sh)):
             # if there is no autogen.sh, automatically fallback to configure
             if os.path.exists(os.path.join(srcdir, 'configure')):
                 self.autogen_sh = 'configure'
@@ -297,6 +300,9 @@ class AutogenModule(Package, DownloadableModule):
 
     do_install.depends = [PHASE_BUILD]
 
+    def skip_install(self, buildscript, last_phase):
+        return self.config.noinstall or self.skip_install_phase
+
     def do_distclean(self, buildscript):
         buildscript.set_action(_('Distcleaning'), self)
         makeargs = self._get_makeargs(buildscript)
@@ -314,6 +320,7 @@ class AutogenModule(Package, DownloadableModule):
                  ('supports-non-srcdir-builds',
                   'supports_non_srcdir_builds', True),
                  ('skip-autogen', 'skip_autogen', False),
+                 ('skip-install', 'skip_install_phase', False),
                  ('autogen-sh', 'autogen_sh', 'autogen.sh'),
                  ('makefile', 'makefile', 'Makefile'),
                  ('supports-static-analyzer', 'supports_static_analyzer', True),
@@ -323,29 +330,15 @@ class AutogenModule(Package, DownloadableModule):
 def parse_autotools(node, config, uri, repositories, default_repo):
     instance = AutogenModule.parse_from_xml(node, config, uri, repositories, default_repo)
 
-    # Make some substitutions; do special handling of '${prefix}' and '${libdir}'
-    prefix_re = re.compile('(\${prefix})')
-    # I'm not sure the replacement of ${libdir} is necessary for firefox...
-    libdir_re = re.compile('(\${libdir})')
-    libsubdir = '/lib'
-    if config.use_lib64:
-        libsubdir = '/lib64'
-
     if node.hasAttribute('autogenargs'):
         autogenargs = node.getAttribute('autogenargs')
-        autogenargs = prefix_re.sub(config.prefix, autogenargs)
-        autogenargs = libdir_re.sub(config.prefix + libsubdir, autogenargs)        
-        instance.autogenargs = autogenargs
+        instance.autogenargs = instance.eval_args(autogenargs)
     if node.hasAttribute('makeargs'):
         makeargs = node.getAttribute('makeargs')
-        makeargs = prefix_re.sub(config.prefix, makeargs)
-        makeargs = libdir_re.sub(config.prefix + libsubdir, makeargs)
-        instance.makeargs = makeargs
+        instance.makeargs = instance.eval_args(makeargs)
     if node.hasAttribute('makeinstallargs'):
         makeinstallargs = node.getAttribute('makeinstallargs')
-        makeinstallargs = prefix_re.sub(config.prefix, makeinstallargs)
-        makeinstallargs = libdir_re.sub(config.prefix + libsubdir, makeinstallargs)
-        instance.makeinstallargs = makeinstallargs
+        instance.makeinstallargs = instance.eval_args(makeinstallargs)
 
     if node.hasAttribute('supports-non-srcdir-builds'):
         instance.supports_non_srcdir_builds = \
@@ -356,6 +349,12 @@ def parse_autotools(node, config, uri, repositories, default_repo):
             instance.skip_autogen = True
         elif skip_autogen == 'never':
             instance.skip_autogen = 'never'
+    if node.hasAttribute('skip-install'):
+        skip_install = node.getAttribute('skip-install')
+        if skip_install.lower() in ('true', 'yes'):
+            instance.skip_install_phase = True
+        else:
+            instance.skip_install_phase = False
 
     if node.hasAttribute('check-target'):
         instance.check_target = (node.getAttribute('check-target') == 'true')
