@@ -44,28 +44,28 @@ class CMakeModule(Package, DownloadableModule):
         Package.__init__(self, name, branch=branch)
         self.cmakeargs = cmakeargs
         self.makeargs  = makeargs
+        self.supports_non_srcdir_builds = True
         self.supports_install_destdir = True
 
+    def eval_args(self, args):
+        args = Package.eval_args(self, args)
+        libsuffix = ''
+        if self.config.use_lib64:
+            libsuffix = '64'
+        args = args.replace('${libsuffix}', libsuffix)
+        return args
+
     def get_srcdir(self, buildscript):
         return self.branch.srcdir
 
     def get_builddir(self, buildscript):
-        if buildscript.config.buildroot:
+        if buildscript.config.buildroot and self.supports_non_srcdir_builds:
             d = buildscript.config.builddir_pattern % (
                 self.branch.checkoutdir or self.branch.get_module_basename())
             return os.path.join(buildscript.config.buildroot, d)
         else:
             return self.get_srcdir(buildscript)
 
-    def eval_args(self, args):
-        args = args.replace('${prefix}', self.config.prefix)
-        libsubdir = 'lib'
-        if self.config.use_lib64:
-            libsubdir = 'lib64'
-        libdir = os.path.join(self.config.prefix, libsubdir)
-        args = args.replace('${libdir}', libdir)
-        return args
-
     def get_cmakeargs(self):
         args = '%s %s' % (self.cmakeargs,
                           self.config.module_cmakeargs.get(
@@ -94,7 +94,13 @@ class CMakeModule(Package, DownloadableModule):
             raise CommandError(_('%s not found') % 'cmake')
         baseargs = '-DCMAKE_INSTALL_PREFIX=%s -DLIB_INSTALL_DIR=%s -Dlibdir=%s' % (
                         prefix, buildscript.config.libdir, buildscript.config.libdir)
-        cmd = 'cmake %s %s %s' % (baseargs, self.get_cmakeargs(), srcdir)
+        cmakeargs = self.get_cmakeargs()
+        # CMake on Windows generates VS projects or NMake makefiles by default.
+        # When using MSYS "MSYS Makefiles" is the best guess. "Unix Makefiles"
+        # and "MinGW Makefiles" could also work (each is a bit different).
+        if os.name == 'nt' and os.getenv("MSYSCON") and '-G' not in cmakeargs:
+            baseargs += ' -G "MSYS Makefiles"'
+        cmd = 'cmake %s %s %s' % (baseargs, cmakeargs, srcdir)
         if os.path.exists(os.path.join(builddir, 'CMakeCache.txt')):
             # remove that file, as it holds the result of a previous cmake
             # configure run, and would be reused unconditionnaly
@@ -149,6 +155,9 @@ class CMakeModule(Package, DownloadableModule):
 
 def parse_cmake(node, config, uri, repositories, default_repo):
     instance = CMakeModule.parse_from_xml(node, config, uri, repositories, default_repo)
+    if node.hasAttribute('supports-non-srcdir-builds'):
+        instance.supports_non_srcdir_builds = \
+                (node.getAttribute('supports-non-srcdir-builds') != 'no')
     if node.hasAttribute('cmakeargs'):
         instance.cmakeargs = node.getAttribute('cmakeargs')
     if node.hasAttribute('makeargs'):
