@@ -64,7 +64,9 @@ class TarballRepository(Repository):
         if name in self.config.branches:
             module = self.config.branches[name]
             if not module:
-                raise FatalError(_('branch for %s has wrong override, check your .jhbuildrc') % name)
+                raise FatalError(_('branch for %(name)s has wrong override, check your %(filename)s') % \
+                                   {'name'     : name,
+                                    'filename' : self.config.filename})
         else:
             if module is None:
                 module = name
@@ -313,12 +315,19 @@ class TarballBranch(Branch):
 
     def checkout(self, buildscript):
         if self.checkout_mode == 'clobber':
-            self._wipedir(buildscript)
+            self._wipedir(buildscript, self.raw_srcdir)
         if not os.path.exists(self.srcdir):
             self._download_and_unpack(buildscript)
         if self.quilt:
             self._quilt_checkout(buildscript)
 
+    def may_checkout(self, buildscript):
+        if os.path.exists(self._local_tarball):
+            return True
+        elif buildscript.config.nonetwork:
+            return False
+        return True
+
     def tree_id(self):
         md5sum = hashlib.md5()
         if self.patches:
